// Global variables

#define NDEBUG 1

 // button pin number
#define BTN_PIN     18
// built in LED number
#define LED_PIN     13
byte debugButtonState = 1;
byte debugOutputValue = 0;

#define D1_SENSOR_PIN   3
byte d1SensorBinaryState = 0;

#define D2_SENSOR_PIN   4
byte d2SensorBinaryState = 0;

#define D3_SENSOR_PIN   5
byte d3SensorBinaryState = 0;

#define D12_SENSOR_PIN   6
byte d12SensorBinaryState = 0;

#define SNST_SENSOR_PIN   11
byte sunsetSensorBinaryState = 0;

#define G1_SWITCH_PIN   13
byte g1SwitchBinaryState = 0;

#define G2_SWITCH_PIN   14
byte g2SwitchBinaryState = 0;

#define G3_SWITCH_PIN   15
byte g3SwitchBinaryState = 0;

byte updatePinOutputFlag = 1;

// Z-Wave channels
ZUNO_SETUP_CHANNELS(
  ZUNO_SWITCH_BINARY(g1SwitchBinaryGetter, g1SwitchBinarySetter),
  ZUNO_SWITCH_BINARY(g2SwitchBinaryGetter, g2SwitchBinarySetter),
  ZUNO_SWITCH_BINARY(g3SwitchBinaryGetter, g3SwitchBinarySetter),
  ZUNO_SENSOR_BINARY(ZUNO_SENSOR_BINARY_TYPE_GENERAL_PURPOSE, sunsetSensorBinaryGetter),
  ZUNO_SENSOR_BINARY(ZUNO_SENSOR_BINARY_TYPE_MOTION, d1SensorBinaryGetter),
  ZUNO_SENSOR_BINARY(ZUNO_SENSOR_BINARY_TYPE_MOTION, d2SensorBinaryGetter),
  ZUNO_SENSOR_BINARY(ZUNO_SENSOR_BINARY_TYPE_MOTION, d3SensorBinaryGetter),
  ZUNO_SENSOR_BINARY(ZUNO_SENSOR_BINARY_TYPE_MOTION, d12SensorBinaryGetter)
);

ZUNO_SETUP_ASSOCIATIONS(ZUNO_ASSOCIATION_GROUP_SET_VALUE);

void setup() {
  pinMode(D1_SENSOR_PIN, INPUT_PULLUP);
  pinMode(D2_SENSOR_PIN, INPUT_PULLUP);
  pinMode(D3_SENSOR_PIN, INPUT_PULLUP);
  pinMode(D12_SENSOR_PIN, INPUT_PULLUP);
  pinMode(SNST_SENSOR_PIN, INPUT_PULLUP);

  pinMode(G1_SWITCH_PIN, OUTPUT);
  pinMode(G2_SWITCH_PIN, OUTPUT);
  pinMode(G3_SWITCH_PIN, OUTPUT);
}

#if NDEBUG
void blink_led() {
    digitalWrite(LED_PIN, HIGH);
    delay(200);
    digitalWrite(LED_PIN, LOW);
    delay(200);
}

void test_loop() {
  byte _debugButtonState = digitalRead(BTN_PIN);
  if (debugButtonState != _debugButtonState) {
    debugButtonState = _debugButtonState;
    if (debugButtonState == 0) {
      if (debugOutputValue) {
        debugOutputValue = LOW;
      } else {
        debugOutputValue = HIGH;
      }
      digitalWrite(G1_SWITCH_PIN, debugOutputValue);
      digitalWrite(G2_SWITCH_PIN, debugOutputValue);
      digitalWrite(G3_SWITCH_PIN, debugOutputValue);
    }
  }
  
  byte _sunsetSensorBinaryState = !digitalRead(SNST_SENSOR_PIN);
  if (sunsetSensorBinaryState != _sunsetSensorBinaryState) {
    sunsetSensorBinaryState = _sunsetSensorBinaryState;
    blink_led();
  }
  
  byte _d1SensorBinaryState = !digitalRead(D1_SENSOR_PIN);
  if (d1SensorBinaryState != _d1SensorBinaryState) {
    d1SensorBinaryState = _d1SensorBinaryState;
    blink_led();
  }
  
  byte _d2SensorBinaryState = !digitalRead(D2_SENSOR_PIN);
  if (d2SensorBinaryState != _d2SensorBinaryState) {
    d2SensorBinaryState = _d2SensorBinaryState;
    blink_led();
  }
  
  byte _d3SensorBinaryState = !digitalRead(D3_SENSOR_PIN);
  if (d3SensorBinaryState != _d3SensorBinaryState) {
    d3SensorBinaryState = _d3SensorBinaryState;
    blink_led();
  }
  
  byte _d12SensorBinaryState = !digitalRead(D12_SENSOR_PIN);
  if (d12SensorBinaryState != _d12SensorBinaryState) {
    d12SensorBinaryState = _d12SensorBinaryState;
    blink_led();
  }
}
#endif


void turnOnLight(byte light_group) {
  switch(light_group) {
    case D1_SENSOR_PIN:
      if (!g2SwitchBinaryState) {
        g2SwitchBinarySetter(ZUNO_SWITCHED_ON);
        zunoSendReport(2);
      }
      break;
    case D2_SENSOR_PIN:
      if (!g1SwitchBinaryState) {
        g1SwitchBinarySetter(ZUNO_SWITCHED_ON);
        zunoSendReport(1);
      }
      if (!g2SwitchBinaryState) {
        g2SwitchBinarySetter(ZUNO_SWITCHED_ON);
        zunoSendReport(2);
      }
      break;
    case D3_SENSOR_PIN:
      if (!g3SwitchBinaryState) {
        g3SwitchBinarySetter(ZUNO_SWITCHED_ON);
        zunoSendReport(3);
      }
      break;
    case D12_SENSOR_PIN:
      if (!g1SwitchBinaryState) {
        g1SwitchBinarySetter(ZUNO_SWITCHED_ON);
        zunoSendReport(1);
      }
      break;
    default:
      break;
  }
}

void turnOffLight(byte light_group) {
  switch(light_group) {
    case D1_SENSOR_PIN:
      if (g2SwitchBinaryState) {
        g2SwitchBinarySetter(ZUNO_SWITCHED_OFF);
        zunoSendReport(2);
      }
      break;
    case D2_SENSOR_PIN:
      if (g1SwitchBinaryState) {
        g1SwitchBinarySetter(ZUNO_SWITCHED_OFF);
        zunoSendReport(1);
      }
      if (g2SwitchBinaryState) {
        g2SwitchBinarySetter(ZUNO_SWITCHED_OFF);
        zunoSendReport(2);
      }
      break;
    case D3_SENSOR_PIN:
      if (g3SwitchBinaryState) {
        g3SwitchBinarySetter(ZUNO_SWITCHED_OFF);
        zunoSendReport(3);
      }
      break;
    case D12_SENSOR_PIN:
      if (g1SwitchBinaryState) {
        g1SwitchBinarySetter(ZUNO_SWITCHED_OFF);
        zunoSendReport(1);
      }
      break;
    default:
      break;
  }
}

void loop() {
  #if NDEBUG
  test_loop();
  #else
  // SNST_SENSOR_PIN 0 - датчик освещенности сработал (закат), отправляем 0xFF
  // SNST_SENSOR_PIN 1 - датчик освещенности в покое (рассвет), отправляем 0x00
  byte _sunsetSensorBinaryState = !digitalRead(SNST_SENSOR_PIN);
  if (sunsetSensorBinaryState != _sunsetSensorBinaryState) {
    sunsetSensorBinaryState = _sunsetSensorBinaryState;
    zunoSendReport(4);
    zunoSendToGroupSetValueCommand(CTRL_GROUP_1, sunsetSensorBinaryState ? ZUNO_SWITCHED_ON : ZUNO_SWITCHED_OFF);
  }
  
  if (sunsetSensorBinaryState) {
    // закат на улице темно проверяем датчики
    // SENSOR_PIN 0 - датчик движения сработал, отправляем 0xFF
    // SENSOR_PIN 1 - датчик движения в покое, отправляем 0x00
    byte _d1SensorBinaryState = !digitalRead(D1_SENSOR_PIN);
    if (d1SensorBinaryState != _d1SensorBinaryState) {
      d1SensorBinaryState = _d1SensorBinaryState;
      zunoSendReport(5);
      if (d1SensorBinaryState) {
        turnOnLight(D1_SENSOR_PIN);
      } else {
        turnOffLight(D1_SENSOR_PIN);
      }
    }
    
    byte _d2SensorBinaryState = !digitalRead(D2_SENSOR_PIN);
    if (d2SensorBinaryState != _d2SensorBinaryState) {
      d2SensorBinaryState = _d2SensorBinaryState;
      zunoSendReport(6);
      if (d2SensorBinaryState) {
        turnOnLight(D2_SENSOR_PIN);
      } else {
        turnOffLight(D2_SENSOR_PIN);
      }
    }
    
    byte _d3SensorBinaryState = !digitalRead(D3_SENSOR_PIN);
    if (d3SensorBinaryState != _d3SensorBinaryState) {
      d3SensorBinaryState = _d3SensorBinaryState;
      zunoSendReport(7);
      if (d3SensorBinaryState) {
        turnOnLight(D3_SENSOR_PIN);
      } else {
        turnOffLight(D3_SENSOR_PIN);
      }
    }
    
    byte _d12SensorBinaryState = !digitalRead(D12_SENSOR_PIN);
    if (d12SensorBinaryState != _d12SensorBinaryState) {
      d12SensorBinaryState = _d12SensorBinaryState;
      zunoSendReport(8);
      if (d12SensorBinaryState) {
        turnOnLight(D12_SENSOR_PIN);
      } else {
        turnOffLight(D12_SENSOR_PIN);
      }
    }
  }

  if (updatePinOutputFlag) {
    updatePinOutputFlag = 0;
    digitalWrite(G1_SWITCH_PIN, g1SwitchBinaryState ? HIGH : LOW);
    digitalWrite(G2_SWITCH_PIN, g2SwitchBinaryState ? HIGH : LOW);
    digitalWrite(G3_SWITCH_PIN, g3SwitchBinaryState ? HIGH : LOW);
  }
  #endif
}

// Getters and setters

byte d1SensorBinaryGetter() {
  return d1SensorBinaryState;
}

byte d2SensorBinaryGetter() {
  return d2SensorBinaryState;
}

byte d3SensorBinaryGetter() {
  return d3SensorBinaryState;
}

byte d12SensorBinaryGetter() {
  return d12SensorBinaryState;
}

byte sunsetSensorBinaryGetter() {
  return sunsetSensorBinaryState;
}

void g1SwitchBinarySetter(byte value) {
  g1SwitchBinaryState = value;
  updatePinOutputFlag = 1;
}

byte g1SwitchBinaryGetter() {
  return g1SwitchBinaryState;
}

void g2SwitchBinarySetter(byte value) {
  g2SwitchBinaryState = value;
  updatePinOutputFlag = 1;
}

byte g2SwitchBinaryGetter() {
  return g2SwitchBinaryState;
}

void g3SwitchBinarySetter(byte value) {
  g3SwitchBinaryState = value;
  updatePinOutputFlag = 1;
}

byte g3SwitchBinaryGetter() {
  return g3SwitchBinaryState;
}
